/*
 * $Id: BindingDemo.java,v 1.3 2005/06/06 14:25:36 rbair Exp $
 *
 * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
 * Santa Clara, California 95054, U.S.A. All rights reserved.
 */

package org.jdesktop.demo.binding;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractListModel;
import javax.swing.JMenuBar;
import org.jdesktop.demo.DemoPanel;
import org.jdesktop.demo.MainWindow;
import org.jdesktop.swingx.JXPanel;

/**
 * Demonstrates various ways of binding UI components to underlying data
 *
 * @author  rbair
 */
public class BindingDemo extends DemoPanel {
    /** A List of each demonstration--all of which are DemoPanel
     * instances. */
    private List<DemoPanel> demoPanels = new ArrayList<DemoPanel>();
    
    /**
     * Creates new form BindingDemo 
     */
    public BindingDemo() {
        demoPanels.add(new DataSetBindingDemoPanel());
        demoPanels.add(new JavaBeanBindingDemoPanel());
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jSplitPane1 = new javax.swing.JSplitPane();
        jSplitPane2 = new javax.swing.JSplitPane();
        jXTitledPanel1 = new org.jdesktop.swingx.JXTitledPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        descriptionPane = new javax.swing.JEditorPane();
        demoContainer = new org.jdesktop.swingx.JXTitledPanel();
        jXTitledPanel3 = new org.jdesktop.swingx.JXTitledPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        demoList = new javax.swing.JList();

        setLayout(new java.awt.GridBagLayout());

        jSplitPane1.setBorder(null);
        jSplitPane1.setDividerLocation(200);
        jSplitPane2.setBorder(null);
        jSplitPane2.setDividerLocation(140);
        jSplitPane2.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jXTitledPanel1.getContentContainer().setLayout(new java.awt.BorderLayout());

        jXTitledPanel1.setTitle("Information");
        jScrollPane2.setBorder(null);
        descriptionPane.setEditable(false);
        descriptionPane.setContentType("text/html");
        jScrollPane2.setViewportView(descriptionPane);

        jXTitledPanel1.getContentContainer().add(jScrollPane2, java.awt.BorderLayout.CENTER);

        jSplitPane2.setLeftComponent(jXTitledPanel1);

        demoContainer.getContentContainer().setLayout(new java.awt.CardLayout());

        demoContainer.setTitle("Demo");
        jSplitPane2.setRightComponent(demoContainer);

        jSplitPane1.setRightComponent(jSplitPane2);

        jXTitledPanel3.getContentContainer().setLayout(new java.awt.BorderLayout());

        jXTitledPanel3.setTitle("Table of Contents");
        jScrollPane1.setBorder(null);
        demoList.setModel(new AbstractListModel() {
            public Object getElementAt(int index) {
                return demoPanels.get(index).getName();
            }

            public int getSize() {
                return demoPanels.size();
            }
        });
        demoList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        demoList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                demoListValueChanged(evt);
            }
        });

        jScrollPane1.setViewportView(demoList);

        jXTitledPanel3.getContentContainer().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jSplitPane1.setLeftComponent(jXTitledPanel3);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(12, 12, 17, 11);
        add(jSplitPane1, gridBagConstraints);

    }
    // </editor-fold>//GEN-END:initComponents

    private void demoListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_demoListValueChanged
        DemoPanel dp = demoPanels.get(demoList.getSelectedIndex());
        descriptionPane.setText(dp.getHtmlDescription());
        demoContainer.setContentContainer(dp.getContent());
        demoContainer.revalidate();
        demoContainer.repaint();
    }//GEN-LAST:event_demoListValueChanged
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        MainWindow.main(new String[]{"-d", "org.jdesktop.demo.binding.BindingDemo"});
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private org.jdesktop.swingx.JXTitledPanel demoContainer;
    private javax.swing.JList demoList;
    private javax.swing.JEditorPane descriptionPane;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JSplitPane jSplitPane2;
    private org.jdesktop.swingx.JXTitledPanel jXTitledPanel1;
    private org.jdesktop.swingx.JXTitledPanel jXTitledPanel3;
    // End of variables declaration//GEN-END:variables

}
