/*
 * HowToWindow.java
 *
 * Created on June 22, 2005, 3:56 PM
 */

package org.jdesktop.demo.swingx;

import java.awt.Cursor;
import java.io.IOException;
import java.net.URL;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import org.jdesktop.swingx.JXErrorDialog;

/**
 *
 * @author  patrick
 */
public class HowToWindow extends javax.swing.JFrame {
    URL currentURL;
    
    /** Creates new form HowToWindow */
    public HowToWindow(final String howToName, final URL howToURL) {
        initComponents();
        setExtendedState(JFrame.MAXIMIZED_BOTH);
        this.setTitle("How To Use: " + howToName);
        SwingUtilities.invokeLater( new Runnable() {
            public void run() {
                String title = HowToWindow.this.getTitle();
                try {
                    HowToWindow.this.setCursor(Cursor.WAIT_CURSOR);
                    HowToWindow.this.setTitle(title + " (loading)");
                    howToPane.setPage(howToURL);
                    currentURL = howToURL;
                } catch (IOException e) {
                    JXErrorDialog.showDialog(null, "Error Loading How-to Page", e);
                } finally {
                    HowToWindow.this.setTitle(title);
                    HowToWindow.this.setCursor(Cursor.DEFAULT_CURSOR);
                }
            }
        });
        howToPane.addHyperlinkListener(linkListener);
    }
    
    private HyperlinkListener linkListener=new HyperlinkListener(){
        public void hyperlinkUpdate(final HyperlinkEvent hle){
            
            if(hle.getEventType()==HyperlinkEvent.EventType.ACTIVATED){
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        try {
                            howToPane.setPage(hle.getURL());
                            currentURL = hle.getURL();
                        } catch (IOException ex) {
                            JXErrorDialog.showDialog(null, "Could not follow that link, sorry.", ex);
                        }
                    }
                });
            }
        }
    };
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jXPanel1 = new org.jdesktop.swingx.JXPanel();
        print = new javax.swing.JButton();
        close = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        howToPane = new javax.swing.JEditorPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        jXPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        print.setMnemonic('P');
        print.setText("Print");
        print.setEnabled(false);
        jXPanel1.add(print);

        close.setMnemonic('C');
        close.setText("Close");
        close.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeActionPerformed(evt);
            }
        });

        jXPanel1.add(close);

        getContentPane().add(jXPanel1, java.awt.BorderLayout.SOUTH);

        howToPane.setEditable(false);
        howToPane.setContentType("text/html");
        jScrollPane1.setViewportView(howToPane);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents
    
    private void closeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeActionPerformed
        this.dispose();
    }//GEN-LAST:event_closeActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new HowToWindow("", getClass().getResource("/org/jdesktop/demo/swingx/resources/howto/JXTableHowTo.html")).setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton close;
    private javax.swing.JEditorPane howToPane;
    private javax.swing.JScrollPane jScrollPane1;
    private org.jdesktop.swingx.JXPanel jXPanel1;
    private javax.swing.JButton print;
    // End of variables declaration//GEN-END:variables
    
}
